@@ -1195,25 +1195,30 @@ shm_tmpfile(void)
 {
 #ifdef SHMDIR
 	int	fd;
-	int	flags;
 	char	template[] = SHMDIR "/shmfd-XXXXXX";
 #ifdef O_TMPFILE
 	fd = open(SHMDIR, O_TMPFILE|O_RDWR|O_CLOEXEC|O_EXCL, 0666);
 	if (fd >= 0) {
-		ErrorF ("Using O_TMPFILE\n");
+		DebugF ("Using O_TMPFILE\n");
 		return fd;
 	}
 	ErrorF ("Not using O_TMPFILE\n");
 #endif
+#ifdef HAVE_MKOSTEMP
+	fd = mkostemp(template, O_CLOEXEC);
+#else
 	fd = mkstemp(template);
+#endif
 	if (fd < 0)
 		return -1;
 	unlink(template);
-	flags = fcntl(fd, F_GETFD);
+#ifndef HAVE_MKOSTEMP
+	int flags = fcntl(fd, F_GETFD);
 	if (flags != -1) {
 		flags |= FD_CLOEXEC;
 		(void) fcntl(fd, F_SETFD, &flags);
 	}
+#endif
 	return fd;
 #else
         return -1;
@@ -1333,7 +1338,7 @@ ProcShmDispatch(ClientPtr client)
     }
 }
 
-static void
+static void _X_COLD
 SShmCompletionEvent(xShmCompletionEvent * from, xShmCompletionEvent * to)
 {
     to->type = from->type;
@@ -1345,7 +1350,7 @@ SShmCompletionEvent(xShmCompletionEvent * from, xShmCompletionEvent * to)
     cpswapl(from->offset, to->offset);
 }
 
-static int
+static int _X_COLD
 SProcShmQueryVersion(ClientPtr client)
 {
     REQUEST(xShmQueryVersionReq);
@@ -1354,7 +1359,7 @@ SProcShmQueryVersion(ClientPtr client)
     return ProcShmQueryVersion(client);
 }
 
-static int
+static int _X_COLD
 SProcShmAttach(ClientPtr client)
 {
     REQUEST(xShmAttachReq);
@@ -1365,7 +1370,7 @@ SProcShmAttach(ClientPtr client)
     return ProcShmAttach(client);
 }
 
-static int
+static int _X_COLD
 SProcShmDetach(ClientPtr client)
 {
     REQUEST(xShmDetachReq);
@@ -1375,7 +1380,7 @@ SProcShmDetach(ClientPtr client)
     return ProcShmDetach(client);
 }
 
-static int
+static int _X_COLD
 SProcShmPutImage(ClientPtr client)
 {
     REQUEST(xShmPutImageReq);
@@ -1396,7 +1401,7 @@ SProcShmPutImage(ClientPtr client)
     return ProcShmPutImage(client);
 }
 
-static int
+static int _X_COLD
 SProcShmGetImage(ClientPtr client)
 {
     REQUEST(xShmGetImageReq);
@@ -1413,7 +1418,7 @@ SProcShmGetImage(ClientPtr client)
     return ProcShmGetImage(client);
 }
 
-static int
+static int _X_COLD
 SProcShmCreatePixmap(ClientPtr client)
 {
     REQUEST(xShmCreatePixmapReq);
@@ -1429,7 +1434,7 @@ SProcShmCreatePixmap(ClientPtr client)
 }
 
 #ifdef SHM_FD_PASSING
-static int
+static int _X_COLD
 SProcShmAttachFd(ClientPtr client)
 {
     REQUEST(xShmAttachFdReq);
@@ -1440,7 +1445,7 @@ SProcShmAttachFd(ClientPtr client)
     return ProcShmAttachFd(client);
 }
 
-static int
+static int _X_COLD
 SProcShmCreateSegment(ClientPtr client)
 {
     REQUEST(xShmCreateSegmentReq);
@@ -1452,7 +1457,7 @@ SProcShmCreateSegment(ClientPtr client)
 }
 #endif  /* SHM_FD_PASSING */
 
-static int
+static int _X_COLD
 SProcShmDispatch(ClientPtr client)
 {
     REQUEST(xReq);
